#include "../../BarinkEngine/src/BarinkEngine.h"
#include "../../BarinkEngine/src/Scene/SceneManager.h"
#include "../../BarinkEngine/src/Scene/SceneNodeTypes.h"
#include "../../BarinkEngine/src/AssetManager/ModelImporter.h"
#include "../../BarinkEngine/src/Graphics/Framebuffer.h"
#include <imgui.h>
#include "GUI.h"
#include "Util.h"

/*
* Define globals
*/
Shader* shader;

char* code = new char[254];

const std::string vertexShaderSource = "build/Debug/test.vs";
const std::string fragmentShaderSource = "build/Debug/test.fs";

BarinkEngine::ModelImporter* MI = new BarinkEngine::ModelImporter();
Scene* Level1;
BarinkEngine::SceneObject* cube;
/*
* Runs once at startup 
* - USe to initialize the game/sandbox/demo
*/
void Start() {

    // Create a level and load it as the current level
    std::string levelName("Test Level");
    Level1 = SceneManager::CreateScene(levelName);
    SceneManager::LoadScene(*Level1);
    
    shader = new Shader(vertexShaderSource, fragmentShaderSource);

    // Create a cube node
   
    cube = MI->Import("build/Debug/Models/cube.obj");
    cube->renderable->material = new Material(*shader);
    cube->renderable->material->Color = glm::vec3(1.0f, 0.0f, 0.0f);

    // What is in cube now ??    
    std::cout <<  "mesh vertices: " << cube->renderable->mesh->vertices.size() << std::endl;
    std::cout << "mesh elements: " << cube->renderable->mesh->elements.size() << std::endl;

    Level1->GetRoot().addChild(*cube);

    memset(code, '\0', 254);
   
    // TODO: Move to runtime/ Engine 
    // NOTE: Submits should later be done through walking the sceneTree
   
    renderer.Submit(cube->renderable);

}

/*
* Runs every frame
* - Use to draw Immediate mode graphics (Not meant for HUD's )
*/
void ImmediateGraphicsDraw() 
{
    // Show internal BarinkEngine stats
    ShowStats();
}

/*
* Runs every frame
* - Meant for game logic ( non-physics related) 
*/
void Update()
{
    
    renderer.Render();
   
}

/*
* Runs at the end of the program
* - Meant for cleanup
*/
void Stop()
{
    delete MI;
    delete shader;
}